#
# raccp.rb
#
#   Copyright (c) 1999,2000 Minero Aoki <aamine@dp.u-netsurf.ne.jp>
#
#   This program is free software.
#   You can distribute/modify this program under the terms of
#   the GNU Lesser General Public License version 2.
#

require 'racc/parser'


module Racc

  class RaccParser < Parser

    def initialize( racc )
      @yydebug     = racc.d_parse && Racc_debug_parser
      @ruletable   = racc.ruletable
      @symboltable = racc.symboltable

      @classname = nil
    end

    attr :classname

    def parse( str )
      @scanner = RaccScanner.new( str )
      @scanner.debug = @yydebug

      do_parse
    end


    private

    def next_token
      @scanner.scan
    end

    def on_error( tok, val, _values )
      if val.respond_to? :id2name then
        v = val.id2name
      elsif String === val then
        v = val
      else
        v = val.inspect
      end
      raise ParseError, "#{@scanner.lineno}: unexpected token '#{v}'"
    end

##### racc 1.2.1 generates ###

racc_reduce_table = [
 0, 0, :racc_error,
 6, 22, :_reduce_1,
 0, 23, :_reduce_none,
 2, 23, :_reduce_none,
 3, 25, :_reduce_4,
 1, 25, :_reduce_none,
 2, 25, :_reduce_6,
 2, 25, :_reduce_7,
 2, 25, :_reduce_8,
 2, 26, :_reduce_9,
 3, 26, :_reduce_10,
 3, 27, :_reduce_11,
 3, 27, :_reduce_12,
 1, 31, :_reduce_none,
 2, 31, :_reduce_none,
 2, 32, :_reduce_15,
 2, 32, :_reduce_16,
 2, 32, :_reduce_17,
 1, 29, :_reduce_18,
 2, 29, :_reduce_19,
 1, 28, :_reduce_20,
 1, 28, :_reduce_21,
 1, 24, :_reduce_22,
 0, 24, :_reduce_none,
 1, 33, :_reduce_24,
 2, 33, :_reduce_25,
 2, 33, :_reduce_26,
 2, 33, :_reduce_27,
 1, 34, :_reduce_none,
 1, 34, :_reduce_29,
 2, 34, :_reduce_30,
 1, 34, :_reduce_31,
 1, 30, :_reduce_32,
 2, 30, :_reduce_33 ]

racc_reduce_n = 34

racc_shift_n = 55

racc_action_table = [
    17,    13,     1,    14,    15,     7,     8,    16,    10,    12,
    42,    17,    17,    43,    44,    45,    47,    48,    16,    16,
    17,    17,    17,    17,    17,    17,    17,    16,    16,    16,
    16,    16,    16,    16,    17,    17,    17,    17,    51,    35,
    52,    16,    16,    16,    16,    41,    20,    23,    24,    25,
    40,    23,    24,    25,    23,    24,    25,    23,    24,    25,
     6,     4,     3,    54 ]

racc_action_check = [
    29,     5,     0,     5,     5,     5,     5,    29,     5,     5,
    28,    25,    37,    29,    29,    29,    29,    29,    25,    37,
    23,     7,    36,    38,    24,    47,    14,    23,     7,    36,
    38,    24,    47,    14,    15,    19,    31,    13,    31,    21,
    32,    15,    19,    31,    13,    27,     8,    27,    27,    27,
    26,    26,    26,    26,    10,    10,    10,    12,    12,    12,
     4,     2,     1,    50 ]

racc_action_pointer = [
     0,    59,    61,   nil,    60,    -3,   nil,    18,    43,   nil,
    41,   nil,    44,    34,    23,    31,   nil,   nil,   nil,    32,
   nil,    36,   nil,    17,    21,     8,    38,    34,     5,    -3,
   nil,    33,    30,   nil,   nil,   nil,    19,     9,    20,   nil,
   nil,   nil,   nil,   nil,   nil,   nil,   nil,    22,   nil,   nil,
    53,   nil,   nil,   nil,   nil ]

racc_action_default = [
   -34,   -34,   -34,    -2,   -34,   -34,    55,   -34,   -34,    -3,
   -34,    -5,   -34,   -23,   -34,   -34,   -21,   -20,   -18,    -7,
   -32,    -8,   -13,   -34,   -34,   -34,   -34,   -34,   -34,   -22,
   -24,   -34,   -34,    -6,   -19,   -33,   -15,   -16,   -17,   -14,
   -11,   -12,    -1,   -26,   -27,   -29,   -25,   -34,   -31,   -28,
   -34,    -4,    -9,   -30,   -10 ]

racc_goto_table = [
    30,    32,    33,    19,     2,    26,    34,    27,    39,    39,
    11,    31,     9,    21,    28,     5,    49,    29,    50,    36,
    37,    38,    46,    34,    34,    34,   nil,   nil,   nil,   nil,
   nil,   nil,   nil,   nil,    53 ]

racc_goto_check = [
     7,     7,     7,     8,     1,    10,     7,    10,    11,    11,
     6,     5,     4,     9,     3,     2,     7,    12,     7,     8,
     8,     8,    13,     7,     7,     7,   nil,   nil,   nil,   nil,
   nil,   nil,   nil,   nil,     7 ]

racc_goto_pointer = [
   nil,     4,    12,     1,     7,    -3,     5,   -13,    -4,     5,
    -5,   -18,     4,    -7 ]

racc_goto_default = [
   nil,   nil,   nil,   nil,   nil,   nil,   nil,    18,   nil,   nil,
   nil,    22,   nil,   nil ]

racc_token_table = {
 false => 0,
 Object.new => 1,
 :XCLASS => 2,
 :XSYMBOL => 3,
 :XRULE => 4,
 :XEND => 5,
 :XCONV => 6,
 :XSTART => 7,
 :XTOKEN => 8,
 :XOPTION => 9,
 :STRING => 10,
 :XPRECHIGH => 11,
 :XPRECLOW => 12,
 :XLEFT => 13,
 :XRIGHT => 14,
 :XNONASSOC => 15,
 ";" => 16,
 ":" => 17,
 "|" => 18,
 "=" => 19,
 :ACTION => 20 }

racc_use_result_var = true

racc_nt_base = 21

Racc_arg = [
 racc_action_table,
 racc_action_check,
 racc_action_default,
 racc_action_pointer,
 racc_goto_table,
 racc_goto_check,
 racc_goto_default,
 racc_goto_pointer,
 racc_nt_base,
 racc_reduce_table,
 racc_token_table,
 racc_shift_n,
 racc_reduce_n,
 racc_use_result_var ]

Racc_debug_parser = false

##### racc system variables end #####


 # reduce 0 omitted

 module_eval( <<'.,.,', '(build)', 83 )
  def _reduce_1( val, _values, result )
      @ruletable.end_register_rule
      @classname = val[1].id2name
   result
  end
.,.,

 # reduce 2 omitted

 # reduce 3 omitted

 module_eval( <<'.,.,', '(build)', 94 )
  def _reduce_4( val, _values, result )
      @symboltable.end_register_conv
   result
  end
.,.,

 # reduce 5 omitted

 module_eval( <<'.,.,', '(build)', 100 )
  def _reduce_6( val, _values, result )
      @ruletable.register_start val[1]
   result
  end
.,.,

 module_eval( <<'.,.,', '(build)', 104 )
  def _reduce_7( val, _values, result )
      @symboltable.register_token val[1]
   result
  end
.,.,

 module_eval( <<'.,.,', '(build)', 108 )
  def _reduce_8( val, _values, result )
      val[1].each do |s|
        @ruletable.register_option s.to_s
      end
   result
  end
.,.,

 module_eval( <<'.,.,', '(build)', 115 )
  def _reduce_9( val, _values, result )
      @symboltable.register_conv( val[0], val[1] )
   result
  end
.,.,

 module_eval( <<'.,.,', '(build)', 119 )
  def _reduce_10( val, _values, result )
      @symboltable.register_conv( val[1], val[2] )
   result
  end
.,.,

 module_eval( <<'.,.,', '(build)', 124 )
  def _reduce_11( val, _values, result )
      @symboltable.end_register_prec( true )
   result
  end
.,.,

 module_eval( <<'.,.,', '(build)', 128 )
  def _reduce_12( val, _values, result )
      @symboltable.end_register_prec( false )
   result
  end
.,.,

 # reduce 13 omitted

 # reduce 14 omitted

 module_eval( <<'.,.,', '(build)', 138 )
  def _reduce_15( val, _values, result )
      @symboltable.register_prec( :Left, val[1] )
   result
  end
.,.,

 module_eval( <<'.,.,', '(build)', 142 )
  def _reduce_16( val, _values, result )
      @symboltable.register_prec( :Right, val[1] )
   result
  end
.,.,

 module_eval( <<'.,.,', '(build)', 146 )
  def _reduce_17( val, _values, result )
      @symboltable.register_prec( :Nonassoc, val[1] )
   result
  end
.,.,

 module_eval( <<'.,.,', '(build)', 151 )
  def _reduce_18( val, _values, result )
      result = val
   result
  end
.,.,

 module_eval( <<'.,.,', '(build)', 155 )
  def _reduce_19( val, _values, result )
      result.push val[1]
   result
  end
.,.,

 module_eval( <<'.,.,', '(build)', 160 )
  def _reduce_20( val, _values, result )
      result = @symboltable.get( result )
   result
  end
.,.,

 module_eval( <<'.,.,', '(build)', 164 )
  def _reduce_21( val, _values, result )
      result = @symboltable.get( eval '"' + val[0] + '"' )
   result
  end
.,.,

 module_eval( <<'.,.,', '(build)', 169 )
  def _reduce_22( val, _values, result )
      unless result.empty? then
        @ruletable.register_rule_from_array result
      end
   result
  end
.,.,

 # reduce 23 omitted

 module_eval( <<'.,.,', '(build)', 178 )
  def _reduce_24( val, _values, result )
      result = val
   result
  end
.,.,

 module_eval( <<'.,.,', '(build)', 182 )
  def _reduce_25( val, _values, result )
      result.push val[1]
   result
  end
.,.,

 module_eval( <<'.,.,', '(build)', 186 )
  def _reduce_26( val, _values, result )
      unless result.empty? then
        @ruletable.register_rule_from_array result
      end
      result.clear
   result
  end
.,.,

 module_eval( <<'.,.,', '(build)', 193 )
  def _reduce_27( val, _values, result )
      pre = result.pop
      unless result.empty? then
        @ruletable.register_rule_from_array result
      end
      result = [pre]
   result
  end
.,.,

 # reduce 28 omitted

 module_eval( <<'.,.,', '(build)', 204 )
  def _reduce_29( val, _values, result )
      result = OrMark.new( @scanner.lineno )
   result
  end
.,.,

 module_eval( <<'.,.,', '(build)', 208 )
  def _reduce_30( val, _values, result )
      result = Prec.new( val[1], @scanner.lineno )
   result
  end
.,.,

 module_eval( <<'.,.,', '(build)', 212 )
  def _reduce_31( val, _values, result )
      result = UserAction.new( *result )
   result
  end
.,.,

 module_eval( <<'.,.,', '(build)', 216 )
  def _reduce_32( val, _values, result )
      result = [ result.id2name ]
   result
  end
.,.,

 module_eval( <<'.,.,', '(build)', 219 )
  def _reduce_33( val, _values, result )
      result.push val[1].id2name
   result
  end
.,.,

 def _reduce_none( val, _values, result )
  result
 end

  end   # class RaccParser

end   # module Racc
